package java.beans;

import java.util.HashMap;
import java.util.IdentityHashMap;
import java.util.Map;
import com.sun.beans.finder.PersistenceDelegateFinder;

public class Encoder
{
    private final PersistenceDelegateFinder finder;
    private Map<Object, Expression> bindings;
    private ExceptionListener exceptionListener;
    boolean executeStatements;
    private Map<Object, Object> attributes;
    
    public Encoder() {
        this.finder = new PersistenceDelegateFinder();
        this.bindings = new IdentityHashMap();
        this.executeStatements = true;
    }
    
    protected void writeObject(Object o) {
        if (o == this) {
            return;
        }
        this.getPersistenceDelegate((o == null) ? null : o.getClass()).writeObject(o, this);
        return;
    }
    
    public void setExceptionListener(ExceptionListener exceptionListener) {
        this.exceptionListener = exceptionListener;
    }
    
    public ExceptionListener getExceptionListener() {
        return (this.exceptionListener != null) ? this.exceptionListener : Statement.defaultExceptionListener;
    }
    
    Object getValue(Expression expression) {
        try {
            return (expression == null) ? null : expression.getValue();
        }
        catch (Exception ex) {
            this.getExceptionListener().exceptionThrown(ex);
            throw new RuntimeException("failed to evaluate: " + expression.toString());
        }
    }
    
    public PersistenceDelegate getPersistenceDelegate(Class<?> clazz) {
        PersistenceDelegate delegate = this.finder.find(clazz);
        if (delegate == null) {
            delegate = MetaData.getPersistenceDelegate(clazz);
            if (delegate != null) {
                this.finder.register(clazz, delegate);
            }
        }
        return delegate;
    }
    
    public void setPersistenceDelegate(Class<?> type, PersistenceDelegate delegate) {
        this.finder.register(type, delegate);
    }
    
    public Object remove(Object o) {
        return this.getValue((Expression)this.bindings.remove(o));
    }
    
    public Object get(Object o) {
        if (o == null || o == this || o.getClass() == String.class) {
            return o;
        }
        return this.getValue((Expression)this.bindings.get(o));
    }
    
    private Object writeObject1(Object o) {
        Object o2 = this.get(o);
        if (o2 == null) {
            this.writeObject(o);
            o2 = this.get(o);
            return o2;
        }
        return o2;
    }
    
    private Statement cloneStatement(Statement statement) {
        Object writeObject1 = this.writeObject1(statement.getTarget());
        Object[] arguments = statement.getArguments();
        Object[] array = new Object[arguments.length];
        for (int i = 0; i < arguments.length; ++i) {
            array[i] = this.writeObject1(arguments[i]);
        }
        Statement statement2 = Statement.class.equals(statement.getClass()) ? new Statement(writeObject1, statement.getMethodName(), array) : new Expression(writeObject1, statement.getMethodName(), array);
        statement2.loader = statement.loader;
        return statement2;
    }
    
    public void writeStatement(Statement statement) {
        Statement cloneStatement = this.cloneStatement(statement);
        if (statement.getTarget() != this && this.executeStatements) {
            try {
                cloneStatement.execute();
            }
            catch (Exception cause) {
                this.getExceptionListener().exceptionThrown(new Exception("Encoder: discarding statement " + cloneStatement, cause));
            }
        }
    }
    
    public void writeExpression(Expression expression) {
        Object value = this.getValue(expression);
        if (this.get(value) != null) {
            return;
        }
        this.bindings.put(value, (Expression)this.cloneStatement(expression));
        this.writeObject(value);
        return;
    }
    
    void clear() {
        this.bindings.clear();
    }
    
    void setAttribute(Object o, Object o2) {
        if (this.attributes == null) {
            this.attributes = new HashMap();
        }
        this.attributes.put(o, o2);
    }
    
    Object getAttribute(Object o) {
        if (this.attributes == null) {
            return null;
        }
        return this.attributes.get(o);
    }
}
